/*
 * Copyright (c) 2004-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GraphGUI.java,v 1.9 2007/06/15 20:10:14 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph.gui;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.List;

import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.graph.Edge;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.GraphPiece;
import edu.indiana.extreme.xbaya.graph.Node;
import edu.indiana.extreme.xbaya.graph.Port;
import edu.indiana.extreme.xbaya.graph.system.MemoNode;
import edu.indiana.extreme.xbaya.graph.util.GraphUtil;

/**
 * @author Satoshi Shirasuna
 */
public class GraphGUI implements GraphPieceGUI {

    private Graph graph;

    /**
     * @param graph
     */
    public GraphGUI(Graph graph) {
        this.graph = graph;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.gui.GraphPieceGUI#mouseClicked(java.awt.event.MouseEvent,
     *      edu.indiana.extreme.xbaya.XBayaEngine)
     */
    public void mouseClicked(MouseEvent event, XBayaEngine engine) {
        GraphPiece piece = getGraphPieceAt(event.getPoint());
        if (piece != null) {
            piece.getGUI().mouseClicked(event, engine);
        }
    }

    /**
     * Gets the bounding Rectangle of this Graph.
     * 
     * @return A rectangle indicating this component's bounds
     */
    protected Rectangle getBounds() {
        Rectangle bounds = new Rectangle();
        for (Node node : this.graph.getNodes()) {
            bounds.add(node.getGUI().getBounds());
        }
        final int margin = 10;
        bounds.height += margin;
        bounds.width += margin;
        return bounds;
    }

    /**
     * @param g
     */
    protected void paint(Graphics2D g) {

        // Calcurate the widge of the nodes.
        for (Node node : this.graph.getNodes()) {
            node.getGUI().calculatePositions(g);
        }

        LinkedList<Node> nodes = new LinkedList<Node>(this.graph.getNodes());
        List<MemoNode> memoNodes = GraphUtil.getNodes(this.graph,
                MemoNode.class);
        nodes.removeAll(memoNodes);

        // Paints the edges before nodes.
        for (Edge edge : this.graph.getEdges()) {
            edge.getGUI().paint(g);
        }

        // Paint regular nodes.
        // The ports are painted from inside of each node.
        for (Node node : nodes) {
            node.getGUI().paint(g);
        }

        // Print memoNodes at last so that they stay on top of everything.
        for (MemoNode node : memoNodes) {
            node.getGUI().paint(g);
        }
    }

    /**
     * Returns the visible object at the specified location. The object is
     * either a Node, a Port, or an Edge.
     * 
     * @param point
     *            The location
     * @return The visible object a the specified location
     */
    protected GraphPiece getGraphPieceAt(Point point) {

        GraphPiece piece = null;

        // Starts from edge because it is drawn first, which means it's at the
        // bottom.
        double minEdgeDist = Double.MAX_VALUE;
        Edge closestEdge = null;
        for (Edge edge : this.graph.getEdges()) {
            double dist = edge.getGUI().getMiddlePosition().distance(point);
            if (dist < minEdgeDist) {
                closestEdge = edge;
                minEdgeDist = dist;
            }
        }
        if (minEdgeDist < 20) {
            piece = closestEdge;
        }

        // Then, each node and ports of it.
        for (Node node : this.graph.getNodes()) {
            // Check the node first
            if (node.getGUI().isIn(point)) {
                piece = node;
            }

            // Find the closest port of this node.
            double minPortDist = Double.MAX_VALUE;
            Port closestPort = null;
            for (Port port : node.getAllPorts()) {
                double dist = port.getGUI().getPosition().distance(point);
                if (dist < minPortDist) {
                    closestPort = port;
                    minPortDist = dist;
                }
            }
            if (minPortDist <= PortGUI.DATA_PORT_SIZE) {
                piece = closestPort;
            }

            // Don't break from this loop because the later ones are drawn at
            // the top of other nodes.
        }
        return piece;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2004-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
